@@ -11,12 +11,14 @@ from sqil_core.utils._utils import fill_gaps, has_at_least_one, make_iterable
 
 from ._core import FitResult, fit_input, fit_output
 from ._guess import (
+    decaying_exp_guess,
     decaying_oscillations_bounds,
     decaying_oscillations_guess,
     gaussian_bounds,
     gaussian_guess,
     lorentzian_bounds,
     lorentzian_guess,
+    many_decaying_oscillations_guess,
     oscillations_bounds,
     oscillations_guess,
 )
@@ -347,19 +349,9 @@ def fit_decaying_exp(
     """
     x, y = x_data, y_data
 
-    # Default initial guess if not provided
-    if guess is None:
-        max_y = np.max(y)
-        min_y = np.min(y)
-        half = 0.5 * (max_y + min_y)
-
-        if y[0] > y[-1]:
-            tau0_idx = np.argmax(y < half)
-        else:
-            tau0_idx = np.argmax(y > half)
-
-        b0 = x[tau0_idx] if tau0_idx != 0 else 0.5 * (x[0] + x[-1])
-        guess = [y[0] - y[-1], b0, y[-1]]
+    # Default intial guess if not provided
+    if has_at_least_one(guess, None):
+        guess = fill_gaps(guess, decaying_exp_guess(x_data, y_data))
 
     # Default bounds if not provided
     if bounds is None:
@@ -619,6 +611,55 @@ def fit_decaying_oscillations(
     return best_fit, metadata
 
 
+@fit_output
+def fit_many_decaying_oscillations(
+    x_data: np.ndarray, y_data: np.ndarray, n: int, guess=None
+):
+    """
+    Fits a sum of `n` exponentially decaying oscillations to the given data.
+
+    Each component of the model is of the form:
+        A_i * exp(-x / tau_i) * cos(2π * T_i * x + phi_i)
+
+    Parameters
+    ----------
+    x_data : np.ndarray
+        1D array of x-values (e.g., time).
+    y_data : np.ndarray
+        1D array of y-values (e.g., signal amplitude).
+    n : int
+        Number of decaying oscillation components to fit.
+    guess : list or None, optional
+        Optional initial parameter guess. If None, a guess is automatically generated
+        using `many_decaying_oscillations_guess`.
+
+    Returns
+    -------
+    FitResult
+    """
+
+    if has_at_least_one(guess, None):
+        guess = fill_gaps(guess, many_decaying_oscillations_guess(x_data, y_data, n))
+
+    res = curve_fit(
+        _models.many_decaying_oscillations,
+        x_data,
+        y_data,
+        p0=guess,
+        # maxfev=10000,
+        full_output=True,
+    )
+
+    metadata = {
+        "param_names": [f"{p}{i}" for i in range(n) for p in ("A", "tau", "phi", "T")]
+        + ["y0"],
+        "predict": lambda x: _models.many_decaying_oscillations(x, *res[0]),
+        "model_name": f"many_decaying_oscillations({n})",
+    }
+
+    return res, metadata
+
+
 @fit_input
 @fit_output
 def fit_oscillations(
